# --------------------------------------------------------------------------------- #
# FLATMENU wxPython IMPLEMENTATION
#
# Andrea Gavana, @ 03 Nov 2006
# Latest Revision: 13 Oct 2008, 19.00 CET
#
#
# TODO List
#
# 1. Work is still in progress, so other functionalities may be added in the future;
# 2. No shadows under MAC, but it may be possible to create them using Carbon.
#
#
# For All Kind Of Problems, Requests Of Enhancements And Bug Reports, Please
# Write To Me At:
#
# gavana@kpo.kz
# andrea.gavana@gmail.com
#
# Or, Obviously, To The wxPython Mailing List!!!
#
#
# End Of Comments
# --------------------------------------------------------------------------------- #

"""
Description
===========

FlatMenu, like the name implies, it is a generic menu implementation.
I tried to provide a full functionality for menus, menubar and toolbar.


FlatMenu supports the following features:

- Fires all the events (UI & Cmd);
- Check items;
- Separators;
- Enabled / Disabled menu items;
- Images on items;
- Toolbar support, with images and separators;
- Controls in toolbar (work in progress);
- Toolbar tools tooltips (done: thanks to Peter Kort);
- Accelerators for menus;
- Accelerators for menubar;
- Radio items in menus;
- Integration with AUI;
- Scrolling when menu is too big to fit the screen (in progress);
- Menu navigation with keyboard;
- Drop down arrow button to the right of the menu, it always contains the
	"Customize" option, which will popup an options dialog. The dialog has the
	following abilities:
	(a) Ability to add/remove menus;
	(b) Select different colour schemes for the menu bar / toolbar;
	(c) Control various options, such as: colour for highlight menu item, draw
		border around menus (classic look only);
	(d) Toolbar floating appearance.
- Allows user to specify grey bitmap for disabled menus/toolbar tools;
- If no grey bitmap is provided, it generates one from the existing bitmap;
- Hidden toolbar items / menu bar items - will appear in a small popmenu
	to the right if they are hidden;
- 4 different colour schemes for the menu bar (more can easily added);
- Scrolling is available if the menu height is greater than the screen height;
- Context menus for menu items;
- Show/hide the drop down arrow which allows the customization of FlatMenu;
- Multiple columns menu window;
- Tooltips for menus and toolbar items on a wx.StatusBar (if present);
- Transparency (alpha channel) for menu windows (for platforms supporting it);
- First attempt in adding controls to FlatToolbar;
- Added a MiniBar (thanks to Vladiuz);
- Added wx.ToolBar methods AddCheckTool/AddRadioTool (thanks to Vladiuz).


Events
======

FlatMenu implements this event for menus and toolbar tools:

- EVT_FLAT_MENU_SELECTED
- EVT_FLAT_MENU_ITEM_MOUSE_OUT
- EVT_FLAT_MENU_ITEM_MOUSE_OVER

Supported Platforms
===================

FlatMenu v0.8 has been tested on the following platforms:
* Windows (Windows XP);
* Linux Ubuntu (Dapper 6.06)
v0.9.* has been tested on
* Windows (Windows XP, Vista);

License And Version:
===================

FlatMenu is freeware and distributed under the wxPython license.
Version 0.9.2

"""

__docformat__ = "epytext"
__version__ = "0.9.2"

import wx
import math

#from fmcustomizedlg import FMCustomizeDlg
from artManager import ArtManager, DCSaver
from fmresources import *


# Some checking to see if we can draw shadows behind the popup menus
# at least on Windows. *REQUIRES* Mark Hammond's win32all extensions
# and ctypes, on Windows obviouly. Mac and GTK have no shadows under
# the menus, and it has been reported that shadows don't work well
# on Windows 2000 and previous.

_libimported = None
_DELAY = 5000

if wx.Platform == "__WXMSW__":
	osVersion = wx.GetOsVersion()
	# Shadows behind menus are supported only in XP
	if osVersion[1] == 5 and osVersion[2] == 1:
		try:
			import win32api
			import win32con
			import winxpgui
			import win32gui
			_libimported = "MH"
		except:
			#try:
			#	import ctypes
			#	_libimported = "ctypes"
			#except:
			pass
	else:
		_libimported = None

# Simple hack, but I don't know how to make it work on Mac
# I don't have  Mac ;-)
#if wx.Platform == "__WXMAC__":
#    try:
#        import ctypes
#        _carbon_dll = ctypes.cdll.LoadLibrary(r'/System/Frameworks/Carbon.framework/Carbon')
#    except:
#        _carbon_dll = None


# FIXME: No way to get shadows on Windows with the original code...
# May anyone share some suggestion on how to make it work??
# Right now I am using win32api to create shadows behind wx.PopupWindow,
# but this will result in *all* the popup windows in an application
# to have shadows behind them, even the user defined wx.PopupWindow
# that do not derive from FlatMenu.

import agw.aui as AUI
#import wx.lib.agw.aui as AUI
AuiPaneInfo = AUI.AuiPaneInfo

# Check for the new method in 2.7 (not present in 2.6.3.3)
if wx.VERSION_STRING < "2.7":
	wx.Rect.Contains = lambda self, point: wx.Rect.Inside(self, point)


wxEVT_FLAT_MENU_DISMISSED = wx.NewEventType()
wxEVT_FLAT_MENU_SELECTED = wx.wxEVT_COMMAND_MENU_SELECTED
wxEVT_FLAT_MENU_ITEM_MOUSE_OVER = wx.NewEventType()
wxEVT_FLAT_MENU_ITEM_MOUSE_OUT = wx.NewEventType()

EVT_FLAT_MENU_DISMISSED = wx.PyEventBinder(wxEVT_FLAT_MENU_DISMISSED, 1)
""" Used internally. """
EVT_FLAT_MENU_SELECTED = wx.PyEventBinder(wxEVT_FLAT_MENU_SELECTED, 2)
""" Fires the wx.EVT_MENU event for FlatMenu. """
EVT_FLAT_MENU_ITEM_MOUSE_OUT = wx.PyEventBinder(wxEVT_FLAT_MENU_ITEM_MOUSE_OUT, 1)
""" Fires an event when the mouse leaves a FlatMenuItem. """
EVT_FLAT_MENU_ITEM_MOUSE_OVER = wx.PyEventBinder(wxEVT_FLAT_MENU_ITEM_MOUSE_OVER, 1)
""" Fires an event when the mouse enters a FlatMenuItem. """


def MSWGetCreateWindowCoords(pos, size):
	""" Creates window coordinates for MS platforms. """

	# yes, those are just some arbitrary hardcoded numbers
	DEFAULT_Y = 200

	if pos.x == -1:

		# if x is set to CW_USEDEFAULT, y parameter is ignored anyhow so we
		# can just as well set it to CW_USEDEFAULT as well
		x = 0
		y = win32con.CW_USEDEFAULT

	else:

		# OTOH, if x is not set to CW_USEDEFAULT, y shouldn't be set to it
		# neither because it is not handled as a special value by Windows then
		# and so we have to choose some default value for it
		x = pos.x
		y = (pos.y == -1 and [DEFAULT_Y] or [pos.y])[0]

	w = WidthDefault(size.x)
	h = HeightDefault(size.y)

	return x, y, w, h


def WidthDefault(w):
	""" Used internally. """

	return (w == -1 and [20] or [w])[0]


def HeightDefault(h):
	""" Used internally. """

	return (h == -1 and [20] or [h])[0]



# ---------------------------------------------------------------------------- #
# Class FlatMenuEvent
# ---------------------------------------------------------------------------- #

class FlatMenuEvent(wx.PyCommandEvent):
	"""
	Event class that supports the FlatMenu-compatible event called
	EVT_FLAT_MENU_SELECTED.
	"""

	def __init__(self, eventType, id=1, nSel=-1, nOldSel=-1):
		""" Default class constructor. """
	
		wx.PyCommandEvent.__init__(self, eventType, id)
		self._eventType = eventType


# ---------------------------------------------------------------------------- #
# Class MenuEntryInfo
# ---------------------------------------------------------------------------- #

class MenuEntryInfo:
	"""
	Internal class which holds information about a menu.
	"""

	def __init__(self, titleOrMenu="", menu=None, state=ControlNormal, cmd=wx.ID_ANY):
		""" Default class constructor. """

		if isinstance(titleOrMenu, basestring):

			self._title = titleOrMenu
			self._menu = menu

			self._rect = wx.Rect()
			self._state = state
			if cmd == wx.ID_ANY:
				cmd = wx.NewId()

			self._cmd = cmd             # the menu itself accelerator id

		else:

			self._title = titleOrMenu._title
			self._menu = titleOrMenu._menu
			self._rect = titleOrMenu._rect
			self._state = titleOrMenu._state
			self._cmd = titleOrMenu._cmd

		self._textBmp = wx.NullBitmap


	def GetTitle(self):
		""" Returns the class title. """

		return self._title


	def GetMenu(self):
		""" Returns the class menu. """

		return self._menu


	def SetRect(self, rect):
		""" Sets the class rect. """

		self._rect = rect


	def GetRect(self):
		""" Returns the class rect. """

		return self._rect


	def SetState(self, state):
		""" Sets the class state. """

		self._state = state


	def GetState(self):
		""" Returns the class state. """

		return self._state


	def SetTextBitmap(self, bmp) :
		""" Sets the class bitmap. """

		self._textBmp = bmp


	def GetTextBitmap(self):
		""" Returns the class bitmap. """

		return self._textBmp


	def GetCmdId(self):
		""" Returns the associated command ID. """

		return self._cmd


# ---------------------------------------------------------------------------- #
# Class StatusBarTimer
# ---------------------------------------------------------------------------- #

class StatusBarTimer(wx.Timer):
	"""Timer used for deleting StatusBar long help after _DELAY seconds."""

	def __init__(self, owner):
		"""
		Default class constructor.
		For internal use: do not call it in your code!
		"""

		wx.Timer.__init__(self)
		self._owner = owner


	def Notify(self):
		"""The timer has expired."""

		self._owner.OnStatusBarTimer()


# ---------------------------------------------------------------------------- #
# Class ShadowPopupWindow
# ---------------------------------------------------------------------------- #

class ShadowPopupWindow(wx.PopupWindow):
	""" Base class for generic FlatMenu derived from wx.PopupWindow. """

	def __init__(self, parent = None):
		""" Default class constructor. """
		if not parent:
			parent = wx.GetApp().GetTopWindow()
		topLevels = wx.GetTopLevelWindows()

		if not parent:
			raise Exception("Can't create menu without parent!")

		wx.PopupWindow.__init__(self, parent)

		if "__WXMSW__" in wx.Platform and _libimported == "MH":

			GCL_STYLE= -26
			cstyle= win32gui.GetClassLong(self.GetHandle(), GCL_STYLE)
			if cstyle & CS_DROPSHADOW == 0:
				win32api.SetClassLong(self.GetHandle(),
									GCL_STYLE, cstyle | CS_DROPSHADOW)

		# popup windows are created hidden by default
		self.Hide()


	def WndProc(self, hWnd, msg, wparam, lparam):
		""" Used internally. """

		# Need not contain anything since the PyCWnd handler
		# processes all messages
		pass



#--------------------------------------------------------
# Class FlatMenuBase
#--------------------------------------------------------

class FlatMenuBase(ShadowPopupWindow):
	"""
	Base class for generic flat menu derived from wx.PopupWindow.
	"""

	def __init__(self, parent = None):
		""" Default class constructor. """

		self._parentMenu = parent
		self._openedSubMenu = None
		self._owner = None
		self._popupPtOffset = 0
		self._showScrollButtons = False
		self._upButton = None
		self._downButton = None
		self._is_dismiss = False

		ShadowPopupWindow.__init__(self, parent)


	def OnDismiss(self):
		""" Fires an event EVT_FLAT_MENU_DISMISSED and handle menu dismiss. """

		# Release mouse capture if needed
		if self.HasCapture():
			self.ReleaseMouse()

		self._is_dismiss = True

		# send an event about our dismissal to the parent (unless we are a sub menu)
		if self.IsShown() and not self._parentMenu:

			event = FlatMenuEvent(wxEVT_FLAT_MENU_DISMISSED, self.GetId())
			event.SetEventObject(self)

			# Send it
			if self.GetMenuOwner():
				self.GetMenuOwner().GetEventHandler().ProcessEvent(event)
			else:
				self.GetEventHandler().ProcessEvent(event)


	def Popup(self, pt, parent):
		"""
		Popups menu at point 'pt'. 'pt' assumed to be in screen coordinates. However,
		if parent is not None, 'pt' is translated into the screen coordinates using
		parent.ClientToScreen().
		"""

		# some controls update themselves from OnIdle() call - let them do it
		wx.GetApp().ProcessIdle()

		# The mouse was pressed in the parent coordinates,
		# e.g. pressing on the left top of a text ctrl
		# will result in (1, 1), these coordinates needs
		# to be converted into screen coords
		self._parentMenu = parent

		# If we are topmost menu, we use the given pt
		# else we use the logical
		# parent (second argument provided to this function)

		if self._parentMenu:
			pos = self._parentMenu.ClientToScreen(pt)
		else:
			pos = pt

		# Fit the menu into screen
		pos = self.AdjustPosition(pos)

		## Is run on some XP systems
		'''
		if self._showScrollButtons:

			sz = self.GetSize()
			# Get the screen height
			scrHeight = wx.SystemSettings_GetMetric(wx.SYS_SCREEN_Y)

			if not self._upButton:
				self._upButton = FlatMenuButton(self, True, ArtManager.Get().GetStockBitmap("arrow_up"))

			if not self._downButton:
				self._downButton = FlatMenuButton(self, False, ArtManager.Get().GetStockBitmap("arrow_down"))

			# position the scrollbar
			self._upButton.SetSize((SCROLL_BTN_HEIGHT, SCROLL_BTN_HEIGHT))
			self._downButton.SetSize((SCROLL_BTN_HEIGHT, SCROLL_BTN_HEIGHT))

			self._upButton.Move((sz.x - SCROLL_BTN_HEIGHT - 4, 4))
			self._downButton.Move((sz.x - SCROLL_BTN_HEIGHT - 4, scrHeight - pos.y - 2 - SCROLL_BTN_HEIGHT))
		'''
		
		self.Move(pos)
		self.Show()

		# Capture mouse event and direct them to us
		self.CaptureMouse()
		self._is_dismiss = False


	def AdjustPosition(self, pos):
		""" Adjusts position so the menu will be fully visible on screen. """

		# Check that the menu can fully appear in the screen
		scrWidth  = wx.SystemSettings_GetMetric(wx.SYS_SCREEN_X)
		scrHeight = wx.SystemSettings_GetMetric(wx.SYS_SCREEN_Y)

		size = self.GetSize()

		# always assume that we have scrollbuttons on
		self._showScrollButtons = False
		pos.y += self._popupPtOffset

		if size.y + pos.y > scrHeight:
			# the menu will be truncated
			if self._parentMenu is None:
				# try to flip the menu
				flippedPosy = pos.y - size.y
				flippedPosy -= self._popupPtOffset

				if flippedPosy >= 0 and flippedPosy + size.y < scrHeight:
					pos.y = flippedPosy
					return pos
				else:
					# We need to popup scrollbuttons!
					self._showScrollButtons = True

			else:
				# we are a submenu
				# try to decrease the y value of the menu position
				newy = pos.y
				newy -= (size.y + pos.y) - scrHeight

				if newy + size.y > scrHeight:
					# probably the menu size is too high to fit
					# the screen, we need scrollbuttons
					self._showScrollButtons = True
				else:
					pos.y = newy

		menuMaxX = pos.x + size.x

		if menuMaxX > scrWidth and pos.x < scrWidth:

			if self._parentMenu:

				# We are submenu
				self._shiftePos = (size.x + self._parentMenu.GetSize().x)
				pos.x -= self._shiftePos
				pos.x += 10

			else:

				self._shiftePos  = ((size.x + pos.x) - scrWidth)
				pos.x -= self._shiftePos

		else:

			if self._parentMenu:
				pos.x += 5

		return pos


	def Dismiss(self, dismissParent, resetOwner):
		""" Dismisses the popup window. """

		# Check if child menu is poped, if so, dismiss it
		if self._openedSubMenu:
			self._openedSubMenu.Dismiss(False, resetOwner)

		self.OnDismiss()

		# Reset menu owner
		if resetOwner:
			self._owner = None

		self.Show(False)

		if self._parentMenu and dismissParent:

			self._parentMenu.OnChildDismiss()
			self._parentMenu.Dismiss(dismissParent, resetOwner)

		self._parentMenu = None


	def OnChildDismiss(self):
		""" Handles children dismiss. """

		self._openedSubMenu = None


	def GetRootMenu(self):
		""" Gets the top level menu. """

		root = self
		while root._parentMenu:
			root = root._parentMenu

		return root


	def SetOwnerHeight(self, height):
		"""
		Sets the menu owner height, this will be used to position the menu below
		or above the owner.
		"""

		self._popupPtOffset = height


	# by default do nothing
	def ScrollDown(self):
		"""
		Scroll one unit down.
		By default this function is empty, let derived class do something.
		"""

		pass


	# by default do nothing
	def ScrollUp(self):
		"""
		Scroll one unit up.
		By default this function is empty, let derived class do something.
		"""

		pass


	def GetMenuOwner(self):
		"""
		Returns the menu logical owner, the owner does not necessarly mean the
		menu parent, it can also be the window that popped up it.
		"""

		return self._owner


#--------------------------------------------------------
# Class FlatMenuItem
#--------------------------------------------------------

class FlatMenuItem:
	"""
	A class that represents an item in a menu.
	"""

	def __init__(self, parent, id=wx.ID_SEPARATOR, text="", helpString="",
				kind=wx.ITEM_NORMAL, subMenu=None, normalBmp=wx.NullBitmap,
				disabledBmp=wx.NullBitmap,
				hotBmp=wx.NullBitmap):
		"""
		Default class constructor.

		Parameters:
		@param parent: menu that the menu item belongs to;
		@param label: text for the menu item, as shown on the menu. An accelerator
					key can be specified using the ampersand '&' character. In
					order to embed an ampersand character in the menu item text,
					the ampersand must be doubled;
		@param kind: may be wx.ITEM_SEPARATOR, wx.ITEM_NORMAL, wx.ITEM_CHECK or
					wx.ITEM_RADIO;
		@param helpString: optional help string that will be shown on the status bar;
		@param normalBmp: normal bitmap to draw to the side of the text, this bitmap
						is used when the menu is enabled;
		@param disabledBmp: 'greyed' bitmap to draw to the side of the text, this
							bitmap is used when the menu is disabled, if none supplied
							normal is used;
		@param hotBmp: hot bitmap to draw to the side of the text, this bitmap is
					used when the menu is hovered, if non supplied, normal is used.
		"""

		self._text = text
		self._kind = kind
		self._helpString = helpString

		if id == wx.ID_ANY:
			id = wx.NewId()

		self._id = id
		self._parentMenu = parent
		self._subMenu = subMenu
		self._normalBmp = normalBmp
		self._disabledBmp = disabledBmp
		self._hotBmp = hotBmp
		self._bIsChecked = False
		self._bIsEnabled = True
		self._mnemonicIdx = wx.NOT_FOUND
		self._isAttachedToMenu = False
		self._accelStr = ""
		self._rect = wx.Rect()
		self._groupPtr = None
		self._visible = False
		self._contextMenu = None

		self.SetLabel(self._text)
		self.SetMenuBar()

		self._checkMarkBmp = wx.BitmapFromXPMData(check_mark_xpm)
		self._checkMarkBmp.SetMask(wx.Mask(self._checkMarkBmp, wx.WHITE))
		self._radioMarkBmp = wx.BitmapFromXPMData(radio_item_xpm)
		self._radioMarkBmp.SetMask(wx.Mask(self._radioMarkBmp, wx.WHITE))


	def SetLongHelp(self, help):

		self._helpString = help


	def GetLongHelp(self):

		return self._helpString


	def GetShortHelp(self):

		return ""


	def Enable(self, enable=True):
		""" Enables or Disables a menu item. """

		self._bIsEnabled = enable
		if self._parentMenu:
			self._parentMenu.UpdateItem(self)


	def GetBitmap(self):
		"""
		Returns the normal bitmap associated to the menu item or wx.NullBitmap if
		not supplied.
		"""

		return self._normalBmp


	def GetDisabledBitmap(self):
		"""
		Returns the disabled bitmap associated to the menu item or wx.NullBitmap
		if not supplied.
		"""

		return self._disabledBmp


	def GetHotBitmap(self):
		"""
		Returns the hot bitmap associated to the menu item or wx.NullBitmap if
		not supplied.
		"""

		return self._hotBmp


	def GetHelp(self):
		""" Returns the item help string. """

		return self._helpString


	def GetId(self):
		""" Returns the item id. """

		return self._id


	def GetKind(self):
		"""
		Returns the menu item kind, can be one of wx.ITEM_SEPARATOR, wx.ITEM_NORMAL,
		wx.ITEM_CHECK or wx.ITEM_RADIO.
		"""

		return self._kind


	def GetLabel(self):
		""" Returns the menu item label (without the accelerator if is part of the string). """

		return self._label


	def GetMenu(self):
		""" Returns the parent menu. """

		return self._parentMenu


	def GetContextMenu(self):
		""" Returns the context menu associated with this item (if any). """

		return self._contextMenu


	def SetContextMenu(self, context_menu):
		""" Assigns a context menu to this item. """

		self._contextMenu = context_menu


	def GetText(self):
		""" Returns the text associated with the menu item including accelerator. """

		return self._text


	def GetSubMenu(self):
		""" Returns the sub-menu. """

		return self._subMenu


	def IsCheckable(self):
		""" Returns True if this item is of type wx.ITEM_CHECK, False otherwise. """

		return self._kind == wx.ITEM_CHECK


	def IsChecked(self):
		""" Returns whether an item is checked or not. """

		return self._bIsChecked


	def IsRadioItem(self):
		""" Returns True if this item is of type wx.ITEM_RADIO, False otherwise. """

		return self._kind == wx.ITEM_RADIO


	def IsEnabled(self):
		""" Returns whether an item is enabled or not. """

		return self._bIsEnabled


	def IsSeparator(self):
		""" Returns True if this item is of type wx.ITEM_SEPARATOR, False otherwise. """

		return self._id == wx.ID_SEPARATOR


	def IsSubMenu(self):
		""" Returns whether an item is a sub-menu or not. """

		return self._subMenu != None


	def SetNormalBitmap(self, bmp):
		""" Sets the menu item normal bitmap. """

		self._normalBmp = bmp


	def SetDisabledBitmap(self, bmp):
		""" Sets the menu item disabled bitmap. """

		self._disabledBmp = bmp


	def SetHotBitmap(self, bmp):
		""" Sets the menu item hot bitmap. """

		self._hotBmp = bmp


	def SetHelp(self, helpString):
		""" Sets the menu item help string. """

		self._helpString = helpString


	def SetMenu(self, menu):
		""" Sets the menu item parent menu. """

		self._parentMenu = menu


	def SetSubMenu(self, menu):
		""" Sets the menu item sub-menu. """

		self._subMenu = menu

		# Fix toolbar update
		self.SetMenuBar()


	def GetAccelString(self):
		""" Returns the accelerator string. """

		return self._accelStr


	def SetRect(self, rect):
		""" Sets the menu item rect. """

		self._rect = rect


	def GetRect(self):
		""" Returns the menu item rect. """

		return self._rect


	def IsShown(self):
		""" Returns whether an item is shown or not. """

		return self._visible


	def Show(self, show=True):
		""" Actually shows the menu item. """

		self._visible = show


	def DrawSelf(self, dc, xCoord, yCoord, imageMarginX, markerMarginX, textX, rightMarginX, selected=False):
		""" Draws the menu item on the specified dc. """

		borderXSize = self._parentMenu.GetBorderXWidth()
		itemHeight = self._parentMenu.GetItemHeight()
		menuWidth  = self._parentMenu.GetMenuWidth()

		artMgr = ArtManager.Get()

		theme = artMgr.GetMenuTheme()

		# Define the item actual rectangle area
		itemRect = wx.Rect(xCoord, yCoord, menuWidth, itemHeight)

		# Define the drawing area
		rect = wx.Rect(xCoord+2, yCoord, menuWidth - 4, itemHeight)

		# Draw the background
		backColor = artMgr.GetMenuFaceColour()
		penColor  = backColor
		backBrush = wx.Brush(backColor)
		lightColour = wx.NamedColour("LIGHT GREY")
		leftMarginWidth = self._parentMenu.GetLeftMarginWidth()

		pen = wx.Pen(penColor)
		dc.SetPen(pen)
		dc.SetBrush(backBrush)
		dc.DrawRectangleRect(rect)

		# Draw the left margin gradient
		self._parentMenu.DrawLeftMargin(dc, itemRect)

		# check if separator
		if self.IsSeparator():

			# Separator is a small grey line separating between
			# menu item. the separator height is 3 pixels
			sepWidth = xCoord + menuWidth - textX - 1
			sepRect1 = wx.Rect(xCoord + textX, yCoord + 1, sepWidth/2, 1)
			sepRect2 = wx.Rect(xCoord + textX + sepWidth/2, yCoord + 1, sepWidth/2-1, 1)

			artMgr.PaintStraightGradientBox(dc, sepRect1, backColor, lightColour, False)
			artMgr.PaintStraightGradientBox(dc, sepRect2, lightColour, backColor, False)
			return

		# Keep the item rect
		self._rect = itemRect

		# Get the bitmap base on the item state (disabled, selected ..)
		bmp = self.GetSuitableBitmap(selected)

		# First we draw the selection rectangle
		if selected:
			artMgr.SetMS2007ButtonSunken(False)
			artMgr.DrawButton(dc, rect, theme, ControlFocus, False)

		if bmp.Ok():

			# Calculate the postion to place the image
			imgHeight = bmp.GetHeight()
			imgWidth  = bmp.GetWidth()

			if imageMarginX == 0:
				xx = rect.x + (leftMarginWidth - imgWidth)/2
			else:
				xx = rect.x + ((leftMarginWidth - rect.height) - imgWidth)/2 + rect.height

			yy = rect.y + (rect.height - imgHeight)/2
			dc.DrawBitmap(bmp, xx, yy, True)

		if self.GetKind() == wx.ITEM_CHECK:

			# Checkable item
			if self.IsChecked():

				# Draw surrounding rectangle around the selection box
				xx = rect.x + 1
				yy = rect.y + 1
				rr = wx.Rect(xx, yy, rect.height-2, rect.height-2)

				if not selected:
					artMgr.SetMS2007ButtonSunken(False)
					artMgr.DrawButton(dc, rr, theme, ControlFocus, False)

				dc.DrawBitmap(self._checkMarkBmp, rr.x + (rr.width - 16)/2, rr.y + (rr.height - 16)/2, True)

		if self.GetKind() == wx.ITEM_RADIO:

			# Checkable item
			if self.IsChecked():

				# Draw surrounding rectangle around the selection box
				xx = rect.x + 1
				yy = rect.y + 1
				rr = wx.Rect(xx, yy, rect.height-2, rect.height-2)

				if not selected:
					artMgr.SetMS2007ButtonSunken(False)
					artMgr.DrawButton(dc, rr, theme, ControlFocus, False)

				dc.DrawBitmap(self._radioMarkBmp, rr.x + (rr.width - 16)/2, rr.y + (rr.height - 16)/2, True)

		# Draw text - without accelerators
		text = self.GetLabel()

		if text:

			font = artMgr.GetFont()
			enabledTxtColour = artMgr.GetTextColourEnable()
			disabledTxtColour = artMgr.GetTextColourDisable()
			textColor = (self.IsEnabled() and [enabledTxtColour] or [disabledTxtColour])[0]

			dc.SetFont(font)
			w, h = dc.GetTextExtent(text)
			dc.SetTextForeground(textColor)

			if self._mnemonicIdx != wx.NOT_FOUND:

				# We divide the drawing to 3 parts
				text1 = text[0:self._mnemonicIdx]
				text2 = text[self._mnemonicIdx]
				text3 = text[self._mnemonicIdx+1:]

				w1, dummy = dc.GetTextExtent(text1)
				w2, dummy = dc.GetTextExtent(text2)
				w3, dummy = dc.GetTextExtent(text3)

				posx = xCoord + textX + borderXSize
				posy = (itemHeight - h)/2 + yCoord

				# Draw first part
				dc.DrawText(text1, posx, posy)

				# mnemonic
				if "__WXGTK__" not in wx.Platform:
					font.SetUnderlined(True)
					dc.SetFont(font)

				posx += w1
				dc.DrawText(text2, posx, posy)

				# last part
				font.SetUnderlined(False)
				dc.SetFont(font)
				posx += w2
				dc.DrawText(text3, posx, posy)

			else:

				w, h = dc.GetTextExtent(text)
				dc.DrawText(text, xCoord + textX + borderXSize, (itemHeight - h)/2 + yCoord)


		# Now draw accelerator
		# Accelerators are aligned to the right
		if self.GetAccelString():

			accelWidth, accelHeight = dc.GetTextExtent(self.GetAccelString())
			dc.DrawText(self.GetAccelString(), xCoord + rightMarginX - accelWidth, (itemHeight - accelHeight)/2 + yCoord)

		# Check if this item has sub-menu - if it does, draw
		# right arrow on the right margin
		if self.GetSubMenu():

			# Draw arrow
			rightArrowBmp = wx.BitmapFromXPMData(menu_right_arrow_xpm)
			rightArrowBmp.SetMask(wx.Mask(rightArrowBmp, wx.WHITE))

			xx = xCoord + rightMarginX + borderXSize
			rr = wx.Rect(xx, rect.y + 1, rect.height-2, rect.height-2)
			dc.DrawBitmap(rightArrowBmp, rr.x + 4, rr.y +(rr.height-16)/2, True)


	def GetHeight(self):
		""" Returns the menu item height. """

		if self.IsSeparator():
			return 3
		else:
			return self._parentMenu._itemHeight


	def GetSuitableBitmap(self, selected):
		""" Gets the bitmap that should be used based on the item state. """

		normalBmp = self._normalBmp
		gBmp = (self._disabledBmp.Ok() and [self._disabledBmp] or [self._normalBmp])[0]
		hotBmp = (self._hotBmp.Ok() and [self._hotBmp] or [self._normalBmp])[0]

		if not self.IsEnabled():
			return gBmp
		elif selected:
			return hotBmp
		else:
			return normalBmp


	def SetLabel(self, text):
		""" Sets the label text for this item from the text (excluding the accelerator). """

		if text:

			indx = text.find("\t")
			if indx >= 0:
				self._accelStr = text[indx+1:]
				label = text[0:indx]
			else:
				self._accelStr = ""
				label = text

			self._mnemonicIdx, self._label = ArtManager.Get().GetAccelIndex(label)

		else:

			self._mnemonicIdx = wx.NOT_FOUND
			self._label = ""

		if self._parentMenu:
			self._parentMenu.UpdateItem(self)


	def SetText(self, text):
		""" Sets the text for this menu item (including accelerators). """

		self._text = text
		self.SetLabel(self._text)


	def SetMenuBar(self):

		# Fix toolbar update
		if self._subMenu and self._parentMenu:
			self._subMenu.SetSubMenuBar(self._parentMenu.GetMenuBarForSubMenu())


	def GetAcceleratorEntry(self):
		""" Returns the accelerator entry associated to this menu item. """

		return wx.GetAccelFromString(self.GetText())


	def GetMnemonicChar(self):
		""" Returns the shortcut char for this menu item. """

		if self._mnemonicIdx == wx.NOT_FOUND:
			return 0

		mnemonic = self._label[self._mnemonicIdx]
		return mnemonic.lower()


	def Check(self, check=True):
		""" Checks or unchecks the menu item. """

		if self.IsRadioItem() and not self._isAttachedToMenu:

			# radio items can be checked only after they are attached to menu
			return

		self._bIsChecked = check

		# update group
		if self.IsRadioItem() and check:
			self._groupPtr.SetSelection(self)

		# Our parent menu might want to do something with this change
		if self._parentMenu:
			self._parentMenu.UpdateItem(self)


#--------------------------------------------------------
# Class FlatMenu
#--------------------------------------------------------

class FlatMenu(FlatMenuBase):
	"""
	A Flat popup menu generic implementation.
	"""

	def __init__(self, parent = None, flyOut = 0):
		""" Default class constructor. """

		self._menuWidth = 2*26
		self._leftMarginWidth = 10
		self._rightMarginWidth = 7
		self._borderXWidth = 2
		self._borderYWidth = 2
		self._activeWin = None
		self._focusWin = None
		self._imgMarginX = 0
		self._markerMarginX = 0
		self._textX = 15
		self._rightMarginPosX = -1
		self._itemHeight = 10
		self._selectedItem = -1
		self._clearCurrentSelection = True
		self._textPadding = 8
		self._marginHeight = 17 ## was 20
		self._marginWidth = 17
		self._accelWidth = 0
		self._mb = None
		self._itemsArr = []
		self._accelArray = []
		self._ptLast = wx.Point()
		self._resizeMenu = True
		self._shiftePos = 0
		self._first = 0
		self._mb_submenu = 0
		self._is_dismiss = False
		self._numCols = 1

		FlatMenuBase.__init__(self, parent)

		self.SetSize(wx.Size(self._menuWidth, self._itemHeight+4))

		self.Bind(wx.EVT_PAINT, self.OnPaint)
		self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBg)
		self.Bind(wx.EVT_MOTION, self.OnMouseMove)
		self.Bind(wx.EVT_ENTER_WINDOW, self.OnMouseEnterWindow)
		self.Bind(wx.EVT_LEAVE_WINDOW, self.OnMouseLeaveWindow)
		self.Bind(wx.EVT_LEFT_DOWN, self.OnMouseLeftDown)
		self.Bind(wx.EVT_LEFT_UP, self.OnMouseLeftUp)
		self.Bind(wx.EVT_LEFT_DCLICK, self.OnMouseLeftDown)
		self.Bind(wx.EVT_RIGHT_DOWN, self.OnMouseRightDown)
		self.Bind(wx.EVT_KILL_FOCUS, self.OnKillFocus)
		self.Bind(wx.EVT_TIMER, self.OnTimer)


	def SetMenuBar(self, mb):
		""" Attaches this menu to a menubar. """

		self._mb = mb


	def SetSubMenuBar(self, mb):

		self._mb_submenu = mb


	def GetMenuBar(self):

		if self._mb_submenu:
			return self._mb_submenu

		return self._mb


	def GetMenuBarForSubMenu(self):

		return self._mb


	def Popup(self, pt, owner=None, parent=None):
		""" Pops up the menu. """

		if "__WXMSW__" in wx.Platform:
			self._mousePtAtStartup = wx.GetMousePosition()

		# each time we popup, need to reset the starting index
		self._first = 0

		# Loop over self menu and send update UI event for
		# every item in the menu
		numEvents = len(self._itemsArr)
		cc = 0
		self._shiftePos = 0

		# Set the owner of the menu. All events will be directed to it.
		# If owner is None, the Default GetParent() is used as the owner
		self._owner = owner

		for cc in xrange(numEvents):
			self.SendUIEvent(cc)

		# Adjust menu position and show it
		FlatMenuBase.Popup(self, pt, parent)

		artMgr = ArtManager.Get()
		artMgr.MakeWindowTransparent(self, artMgr.GetTransparency())

		# Replace the event handler of the active window to direct
		# all keyboard events to us and the focused window to direct char events to us
		self._activeWin = wx.GetActiveWindow()
		if self._activeWin:

			oldHandler = self._activeWin.GetEventHandler()
			newEvtHandler = MenuKbdRedirector(self, oldHandler)
			self._activeWin.PushEventHandler(newEvtHandler)

		if "__WXMSW__" in wx.Platform:
			self._focusWin = wx.Window.FindFocus()
		elif "__WXGTK__" in wx.Platform:
			self._focusWin = self
		else:
			self._focusWin = None

		if self._focusWin:
			newEvtHandler = FocusHandler(self)
			self._focusWin.PushEventHandler(newEvtHandler)


	def Append(self, id, item, helpString, kind):
		""" Appends an item to this menu. """

		newItem = FlatMenuItem(self, id, item, helpString, kind)
		return self.AppendItem(newItem)


	def AppendMenu(self, id, item, subMenu, helpString):
		""" Appends a menu to this menu. """

		newItem = FlatMenuItem(self, id, item, helpString, wx.ITEM_NORMAL, subMenu)
		return self.AppendItem(newItem)


	# The main Append function
	def AppendItem(self, menuItem):
		""" Appends an item to this menu. """

		if not menuItem:
			raise Exception("Adding None item?")
			return

		# Reparent to us
		menuItem.SetMenu(self)
		self._itemsArr.append(menuItem)
		menuItem._isAttachedToMenu = True

		# Update the menu width if necessary
		menuItemWidth = self.GetMenuItemWidth(menuItem)
		self._menuWidth = (self._menuWidth > menuItemWidth + self._accelWidth and \
						[self._menuWidth] or [menuItemWidth + self._accelWidth])[0]

		menuHeight = 0
		switch = 1e6

		if self._numCols > 1:
			nItems = len(self._itemsArr)
			switch = int(math.ceil((nItems - self._first)/float(self._numCols)))

		for indx, item in enumerate(self._itemsArr):

			if indx >= switch:
				break

			if item.IsSeparator():
				menuHeight += 3
			else:
				menuHeight += self._itemHeight

		self.SetSize(wx.Size(self._menuWidth*self._numCols, menuHeight+4))

		# Add accelerator entry to the menu if needed
		accel = menuItem.GetAcceleratorEntry()

		if accel:
			accel.Set(accel.GetFlags(), accel.GetKeyCode(), menuItem.GetId())
			self._accelArray.append(accel)

		self.UpdateRadioGroup(menuItem)

		return menuItem


	def GetMenuItemWidth(self, menuItem):
		""" Returns the width of a particular item. """

		menuItemWidth = 0
		text = menuItem.GetLabel() # Without accelerator
		accel = menuItem.GetAccelString()

		dc = wx.ClientDC(self)

		font = ArtManager.Get().GetFont()
		dc.SetFont(font)

		accelFiller = "XXXX"     # 4 spaces betweem text and accel column

		# Calc text length/height
		dummy, itemHeight = dc.GetTextExtent("Tp")
		width, height = dc.GetTextExtent(text)
		accelWidth, accelHeight = dc.GetTextExtent(accel)
		filler, dummy = dc.GetTextExtent(accelFiller)

		bmpHeight = bmpWidth = 0

		if menuItem.GetBitmap().Ok():
			bmpHeight = menuItem.GetBitmap().GetHeight()
			bmpWidth  = menuItem.GetBitmap().GetWidth()

		if itemHeight < self._marginHeight:
			itemHeight = self._marginHeight

		itemHeight = (bmpHeight > self._itemHeight and [bmpHeight] or [itemHeight])[0]
		itemHeight += 2*self._borderYWidth

		# Update the global menu item height if needed
		self._itemHeight = (self._itemHeight > itemHeight and [self._itemHeight] or [itemHeight])[0]
		self._marginWidth = (self._marginWidth > bmpWidth and [self._marginWidth] or [bmpWidth])[0]

		# Update the accel width
		accelWidth += filler
		if accel:
			self._accelWidth = (self._accelWidth > accelWidth and [self._accelWidth] or [accelWidth])[0]

		# In case the item has image & is type radio or check, we need double size
		# left margin
		factor = (((menuItem.GetBitmap() != wx.NullBitmap) and \
				(menuItem.IsCheckable() or (menuItem.GetKind() == wx.ITEM_RADIO))) and [2] or [1])[0]

		if factor == 2:

			self._imgMarginX = self._marginWidth + 2*self._borderXWidth
			self._leftMarginWidth = 2 * self._marginWidth + 2*self._borderXWidth

		else:

			self._leftMarginWidth = ((self._leftMarginWidth > self._marginWidth + 2*self._borderXWidth) and \
									[self._leftMarginWidth] or [self._marginWidth + 2*self._borderXWidth])[0]

		menuItemWidth = self.GetLeftMarginWidth() + 2*self.GetBorderXWidth() + width + self.GetRightMarginWidth()
		self._textX = self._imgMarginX + self._marginWidth + self._textPadding

		# update the rightMargin X position
		self._rightMarginPosX = ((self._textX + width + self._accelWidth> self._rightMarginPosX) and \
								[self._textX + width + self._accelWidth] or [self._rightMarginPosX])[0]

		return menuItemWidth


	def GetMenuWidth(self):
		""" Returns the menu width. """

		return self._menuWidth


	def GetLeftMarginWidth(self):
		""" Returns the menu left margin width. """

		return self._leftMarginWidth


	def GetRightMarginWidth(self):
		""" Returns the menu right margin width. """

		return self._rightMarginWidth


	def GetBorderXWidth(self):
		""" Returns the menu border x-width. """

		return self._borderXWidth


	def GetBorderYWidth(self):
		""" Returns the menu border y-width. """

		return self._borderYWidth


	def GetItemHeight(self):
		""" Returns the height of a particular item. """

		return self._itemHeight


	def AppendCheckItem(self, id, item, helpString):
		""" Appends a wx.ITEM_CHECK item to this menu. """

		newItem = FlatMenuItem(self, id, item, helpString, wx.ITEM_CHECK)
		return self.AppendItem(newItem)


	def AppendRadioItem(self, id, item, helpString):
		""" Appends a wx.ITEM_RADIO item to this menu. """

		newItem = FlatMenuItem(self, id, item, helpString, wx.ITEM_RADIO)
		return self.AppendItem(newItem)


	def AppendSeparator(self):
		""" Appends a wx.ITEM_SEPARATOR item to this menu. """

		newItem = FlatMenuItem(self)
		return self.AppendItem(newItem)


	def InsertSeparator(self, pos):
		""" Inserts a wx.ITEM_SEPARATOR item in teh position 'pos'. """

		newItem = FlatMenuItem(self)
		return self.Insert(pos, newItem)


	def Dismiss(self, dismissParent, resetOwner):
		""" Dismisses the popup window. """

		if self._activeWin:

			self._activeWin.PopEventHandler(True)
			self._activeWin = None

		if self._focusWin:

			self._focusWin.PopEventHandler(True)
			self._focusWin = None

		self._selectedItem = -1

		if self._mb:
			self._mb.RemoveHelp()

		FlatMenuBase.Dismiss(self, dismissParent, resetOwner)


	def OnPaint(self, event):
		""" Handles the wx.EVT_PAINT event for FlatMenu. """

		dc = wx.PaintDC(self)
		self.DoDrawMenu(dc)

		# We need to redraw all our child menus
		self.RefreshChilds()


	def UpdateItem(self, item):
		""" Updates an item. """

		# notify menu bar that an item was modified directly
		if item and self._mb:
			self._mb.UpdateItem(item)


	def OnEraseBg(self, event):
		""" Handles the wx.EVT_ERASE_BACKGROUND event for FlatMenu. """

		pass


	def DoDrawMenu(self, dc):
		""" Actually draws the menu. """

		menuRect = self.GetMenuRect()
		menuBmp = wx.EmptyBitmap(menuRect.width, menuRect.height)

		mem_dc = wx.MemoryDC()
		mem_dc.SelectObject(menuBmp)

		# color the menu face with background color
		backColor = ArtManager.Get().GetMenuFaceColour()
		penColor  = wx.SystemSettings_GetColour(wx.SYS_COLOUR_BTNSHADOW)

		backBrush = wx.Brush(backColor)
		pen = wx.Pen(penColor)

		mem_dc.SetPen(pen)
		mem_dc.SetBrush(backBrush)
		mem_dc.DrawRectangleRect(menuRect)

		# draw items
		posy = 2
		nItems = len(self._itemsArr)

		# make all items as non-visible first
		for item in self._itemsArr:
			item.Show(False)

		visibleItems = 0
		screenHeight = wx.SystemSettings_GetMetric(wx.SYS_SCREEN_Y)

		numCols = self.GetNumberColumns()
		switch, posx, index = 1e6, 0, 0
		if numCols > 1:
			switch = int(math.ceil((nItems - self._first)/float(numCols)))

		for nCount in xrange(self._first, nItems):

			visibleItems += 1
			item = self._itemsArr[nCount]
			item.DrawSelf(mem_dc,
						posx,
						posy,
						self._imgMarginX,
						self._markerMarginX,
						self._textX,
						self._rightMarginPosX,
						nCount == self._selectedItem
						)
			posy += item.GetHeight()
			item.Show()

			if visibleItems >= switch:
				posy = 2
				index += 1
				posx = self._menuWidth*index
				visibleItems = 0

			# make sure we draw only visible items
			pp = self.ClientToScreen(wx.Point(0, posy))

			if pp.y > screenHeight:
				break

		if self._showScrollButtons:
			if self._upButton:
				self._upButton.Draw(mem_dc)
			if self._downButton:
				self._downButton.Draw(mem_dc)

		dc.Blit(0, 0, menuBmp.GetWidth(), menuBmp.GetHeight(), mem_dc, 0, 0)


	def DrawSelection(self, dc, oldSelection=-1):
		""" Redraws the menu. """

		self.Refresh()


	def RefreshChilds(self):
		"""
		In some cases, we need to perform a recursive refresh for all opened submenu
		from this.
		"""

		# Draw all childs menus of self menu as well
		child = self._openedSubMenu
		while child:
			dc = wx.ClientDC(child)
			child.DoDrawMenu(dc)
			child = child._openedSubMenu


	def DrawLeftMargin(self, dc, menuRect):
		""" Draws the menu left margin. """

		# Construct the margin rectangle
		marginRect = wx.Rect(menuRect.x+1, menuRect.y, self.GetLeftMarginWidth(), menuRect.height)

		# Set the gradient colors
		artMgr = ArtManager.Get()
		faceColour = artMgr.GetMenuFaceColour()

		if marginRect:#Style2007 == artMgr.GetMenuTheme():
			##
			dcsaver = DCSaver(dc)
			#marginColor = artMgr.DarkColour(faceColour, 5)
			#dc.SetPen(wx.Pen(marginColor))
			#dc.SetBrush(wx.Brush(marginColor))
			#dc.DrawRectangleRect(marginRect)

			#dc.SetPen(wx.WHITE_PEN)
			#dc.DrawLine(marginRect.x , marginRect.y, marginRect.x , marginRect.y + marginRect.height)

			borderColor = artMgr.DarkColour(faceColour, 20)
			dc.SetPen(wx.Pen(borderColor))
			dc.DrawLine(marginRect.x -2, marginRect.y, marginRect.x -2, marginRect.y + marginRect.height)

		elif self.flyOut != 1:  ## Flyout

			startColor = artMgr.DarkColour(faceColour, 20)
			endColor   = faceColour
			artMgr.PaintStraightGradientBox(dc, marginRect, startColor, endColor, False)


	def GetMenuRect(self):
		""" Returns the menu rect. """

		clientRect = self.GetClientRect()
		return wx.Rect(clientRect.x, clientRect.y, clientRect.width, clientRect.height)


	def OnKeyDown(self, event):
		""" Handles the wx.EVT_KEY_DOWN event for FlatMenu. """

		self.OnChar(event.GetKeyCode())


	def OnChar(self, key):
		""" Handles key events for FlatMenu. """

		processed = True

		if key == wx.WXK_ESCAPE:

			if self._parentMenu:
				self._parentMenu.CloseSubMenu(-1)
			else:
				self.Dismiss(True, True)

		elif key == wx.WXK_LEFT:

			if self._parentMenu:
				# We are a submenu, dismiss us.
				self._parentMenu.CloseSubMenu(-1)
			else:
				# try to find our root menu, if we are attached to menubar,
				# let it try and open the previous menu
				root = self.GetRootMenu()
				if root:
					if root._mb:
						root._mb.ActivatePreviousMenu()

		elif key == wx.WXK_RIGHT:

			if not self.TryOpenSubMenu(self._selectedItem, True):
				# try to find our root menu, if we are attached to menubar,
				# let it try and open the previous menu
				root = self.GetRootMenu()
				if root:
					if root._mb:
						root._mb.ActivateNextMenu()

		elif key == wx.WXK_UP:
			self.AdvanceSelection(False)

		elif key == wx.WXK_DOWN:

			self.AdvanceSelection()

		elif key in [wx.WXK_RETURN, wx.WXK_NUMPAD_ENTER]:
			self.DoAction(self._selectedItem)

		elif key == wx.WXK_HOME:

			# Select first item of the menu
			if self._selectedItem != 0:
				oldSel = self._selectedItem
				self._selectedItem = 0
				dc = wx.ClientDC(self)
				self.DrawSelection(dc, oldSel)

		elif key == wx.WXK_END:

			# Select last item of the menu
			if self._selectedItem != len(self._itemsArr)-1:
				oldSel = self._selectedItem
				self._selectedItem = len(self._itemsArr)-1
				dc = wx.ClientDC(self)
				self.DrawSelection(dc, oldSel)

		elif key in [wx.WXK_CONTROL, wx.WXK_ALT]:
			# Alt was pressed
			root = self.GetRootMenu()
			root.Dismiss(False, True)

		else:
			try:
				chrkey = chr(key)
			except:
				return processed

			if chrkey.isalnum():

				ch = chrkey.lower()

				# Iterate over all the menu items
				itemIdx = -1
				occur = 0

				for i in xrange(len(self._itemsArr)):

					item = self._itemsArr[i]
					mnemonic = item.GetMnemonicChar()

					if mnemonic == ch:

						if itemIdx == -1:

							itemIdx = i
							# We keep the index of only
							# the first occurence

						occur += 1

						# Keep on looping until no more items for self menu

				if itemIdx != -1:

					if occur > 1:

						# We select the first item
						if self._selectedItem == itemIdx:
							return processed

						oldSel = self._selectedItem
						self._selectedItem = itemIdx
						dc = wx.ClientDC(self)
						self.DrawSelection(dc, oldSel)

					elif occur == 1:

						# Activate the item, if self is a submenu item we first select it
						item = self._itemsArr[itemIdx]
						if item.IsSubMenu() and self._selectedItem != itemIdx:

							oldSel = self._selectedItem
							self._selectedItem = itemIdx
							dc = wx.ClientDC(self)
							self.DrawSelection(dc, oldSel)

						self.DoAction(itemIdx)

				else:

					processed = False

		return processed


	def AdvanceSelection(self, down=True):
		""" Advance forward or backward the current selection. """

		# make sure we have at least two items in the menu (which are not
		# separators)
		num=0
		singleItemIdx = -1

		for i in xrange(len(self._itemsArr)):

			item = self._itemsArr[i]
			if item.IsSeparator():
				continue
			num += 1
			singleItemIdx = i

		if num < 1:
			return

		if num == 1:
			# Select the current one
			self._selectedItem = singleItemIdx
			dc = wx.ClientDC(self)
			self.DrawSelection(dc, -1)
			return

		oldSelection = self._selectedItem

		if not down:

			# find the next valid item
			while 1:

				self._selectedItem -= 1
				if self._selectedItem < 0:
					self._selectedItem = len(self._itemsArr)-1
				if not self._itemsArr[self._selectedItem].IsSeparator():
					break

		else:

			# find the next valid item
			while 1:

				self._selectedItem += 1
				if self._selectedItem > len(self._itemsArr)-1:
					self._selectedItem = 0
				if not self._itemsArr[self._selectedItem].IsSeparator():
					break

		dc = wx.ClientDC(self)
		self.DrawSelection(dc, oldSelection)


	def HitTest(self, pos):
		""" HitTest method for FlatMenu. """

		if self._showScrollButtons:

			if self._upButton and self._upButton.GetClientRect().Contains(pos):
				return MENU_HT_SCROLL_UP, -1

			if self._downButton and self._downButton.GetClientRect().Contains(pos):
				return MENU_HT_SCROLL_DOWN, -1

		for ii, item in enumerate(self._itemsArr):

			if item.GetRect().Contains(pos) and item.IsEnabled() and item.IsShown():
				return MENU_HT_ITEM, ii

		return MENU_HT_NONE, -1


	def OnMouseMove(self, event):
		""" Handles the wx.EVT_MOTION event for FlatMenu. """

		if "__WXMSW__" in wx.Platform:
			# Ignore dummy mouse move events
			pt = wx.GetMousePosition()
			if self._mousePtAtStartup == pt:
				return

		pos = event.GetPosition()

		# we need to ignore extra mouse events: example when this happens is when
		# the mouse is on the menu and we open a submenu from keyboard - Windows
		# then sends us a dummy mouse move event, we (correctly) determine that it
		# happens in the parent menu and so immediately close the just opened
		# submenunot

		if "__WXMSW__" in wx.Platform:

			ptCur = self.ClientToScreen(pos)
			if ptCur == self._ptLast:
				return

			self._ptLast = ptCur

		# first let the scrollbar handle it
		self.TryScrollButtons(event)
		self.ProcessMouseMove(pos)


	def OnMouseLeftDown(self, event):
		""" Handles the wx.EVT_LEFT_DOWN event for FlatMenu. """

		if self.TryScrollButtons(event):
			return

		pos = event.GetPosition()
		self.ProcessMouseLClick(pos)


	def OnMouseLeftUp(self, event):
		""" Handles the wx.EVT_LEFT_UP event for FlatMenu. """

		if self.TryScrollButtons(event):
			return

		pos = event.GetPosition()
		rect = self.GetClientRect()

		if not rect.Contains(pos):

			# The event is not in our coords,
			# so we try our parent
			win = self._parentMenu

			while win:

				# we need to translate our client coords to the client coords of the
				# window we forward this event to
				ptScreen = self.ClientToScreen(pos)
				p = win.ScreenToClient(ptScreen)

				if win.GetClientRect().Contains(p):

					event.m_x = p.x
					event.m_y = p.y
					win.OnMouseLeftUp(event)
					return

				else:
					# try the grandparent
					win = win._parentMenu

		else:
			self.ProcessMouseLClickEnd(pos)

		if self._showScrollButtons:

			if self._upButton:
				self._upButton.ProcessLeftUp(pos)
			if self._downButton:
				self._downButton.ProcessLeftUp(pos)


	def OnMouseRightDown(self, event):
		""" Handles the wx.EVT_RIGHT_DOWN event for FlatMenu. """

		if self.TryScrollButtons(event):
			return

		pos = event.GetPosition()
		self.ProcessMouseRClick(pos)


	def ProcessMouseRClick(self, pos):
		""" Processes mouse right clicks. """

		rect = self.GetClientRect()

		if not rect.Contains(pos):

			# The event is not in our coords,
			# so we try our parent

			win = self._parentMenu
			while win:

				# we need to translate our client coords to the client coords of the
				# window we forward self event to
				ptScreen = self.ClientToScreen(pos)
				p = win.ScreenToClient(ptScreen)

				if win.GetClientRect().Contains(p):
					win.ProcessMouseRClick(p)
					return

				else:
					# try the grandparent
					win = win._parentMenu

			# At this point we can assume that the event was not
			# processed, so we dismiss the menu and its children
			self.Dismiss(True, True)
			return

		# test if we are on a menu item
		res, itemIdx = self.HitTest(pos)
		if res == MENU_HT_ITEM:
			self.OpenItemContextMenu(itemIdx)


	def OpenItemContextMenu(self, itemIdx):
		""" Open an item's context menu (if any). """

		item = self._itemsArr[itemIdx]
		context_menu = item.GetContextMenu()

		# If we have a context menu, close any opened submenu
		if context_menu:
			self.CloseSubMenu(itemIdx, True)

		if context_menu and not context_menu.IsShown():
			# Popup child menu
			pos = wx.Point()
			pos.x = item.GetRect().GetWidth() + item.GetRect().GetX() - 5
			pos.y = item.GetRect().GetY()
			self._clearCurrentSelection = False
			self._openedSubMenu = context_menu
			context_menu.Popup(self.ScreenToClient(wx.GetMousePosition()), self._owner, self)
			return True

		return False


	def ProcessMouseLClick(self, pos):
		""" Processes mouse left clicks. """

		rect = self.GetClientRect()

		if not rect.Contains(pos):

			# The event is not in our coords,
			# so we try our parent

			win = self._parentMenu
			while win:

				# we need to translate our client coords to the client coords of the
				# window we forward self event to
				ptScreen = self.ClientToScreen(pos)
				p = win.ScreenToClient(ptScreen)

				if win.GetClientRect().Contains(p):
					win.ProcessMouseLClick(p)
					return

				else:
					# try the grandparent
					win = win._parentMenu

			# At this point we can assume that the event was not
			# processed, so we dismiss the menu and its children
			self.Dismiss(True, True)
			return


	def ProcessMouseLClickEnd(self, pos):
		""" Processes mouse left clicks. """

		self.ProcessMouseLClick(pos)

		# test if we are on a menu item
		res, itemIdx = self.HitTest(pos)

		if res == MENU_HT_ITEM:
			self.DoAction(itemIdx)

		elif res == MENU_HT_SCROLL_UP:
			if self._upButton:
				self._upButton.ProcessLeftDown(pos)

		elif res == MENU_HT_SCROLL_DOWN:
			if self._downButton:
				self._downButton.ProcessLeftDown(pos)

		else:
			self._selectedItem = -1


	def ProcessMouseMove(self, pos):
		""" Processes mouse movements. """

		rect = self.GetClientRect()

		if not rect.Contains(pos):

			# The event is not in our coords,
			# so we try our parent

			win = self._parentMenu
			while win:

				# we need to translate our client coords to the client coords of the
				# window we forward self event to
				ptScreen = self.ClientToScreen(pos)
				p = win.ScreenToClient(ptScreen)

				if win.GetClientRect().Contains(p):
					win.ProcessMouseMove(p)
					return

				else:
					# try the grandparent
					win = win._parentMenu

			# If we are attached to a menu bar,
			# let him process the event as well
			if self._mb:

				ptScreen = self.ClientToScreen(pos)
				p = self._mb.ScreenToClient(ptScreen)

				if self._mb.GetClientRect().Contains(p):

					# let the menu bar process it
					self._mb.ProcessMouseMoveFromMenu(p)
					return

			if self._mb_submenu:
				ptScreen = self.ClientToScreen(pos)
				p = self._mb_submenu.ScreenToClient(ptScreen)
				if self._mb_submenu.GetClientRect().Contains(p):
					# let the menu bar process it
					self._mb_submenu.ProcessMouseMoveFromMenu(p)
					return

			return

		# test if we are on a menu item
		res, itemIdx = self.HitTest(pos)

		if res == MENU_HT_SCROLL_DOWN:

			if self._downButton:
				self._downButton.ProcessMouseMove(pos)

		elif res == MENU_HT_SCROLL_UP:

			if self._upButton:
				self._upButton.ProcessMouseMove(pos)

		elif res == MENU_HT_ITEM:

			if self._downButton:
				self._downButton.ProcessMouseMove(pos)

			if self._upButton:
				self._upButton.ProcessMouseMove(pos)

			if self._selectedItem == itemIdx:
				return

			# Message to send when out of last selected item
			if self._selectedItem != -1:
				self.SendOverItem(self._selectedItem, False)
			self.SendOverItem(itemIdx, True)   # Message to send when over an item

			oldSelection = self._selectedItem
			self._selectedItem = itemIdx
			self.CloseSubMenu(self._selectedItem)

			dc = wx.ClientDC(self)
			self.DrawSelection(dc, oldSelection)

			self.TryOpenSubMenu(self._selectedItem)

			if self._mb:
				self._mb.RemoveHelp()
				if itemIdx >= 0:
					self._mb.DoGiveHelp(self._itemsArr[itemIdx])

		else:

			# Message to send when out of last selected item
			if self._selectedItem != -1:
				item = self._itemsArr[self._selectedItem]
				if item.IsSubMenu() and item.GetSubMenu().IsShown():
					return

				# Message to send when out of last selected item
				if self._selectedItem != -1:
					self.SendOverItem(self._selectedItem, False)

			oldSelection = self._selectedItem
			self._selectedItem = -1
			dc = wx.ClientDC(self)
			self.DrawSelection(dc, oldSelection)


	def OnMouseLeaveWindow(self, event):
		""" Handles the wx.EVT_LEAVE_WINDOW event for FlatMenu. """

		if self._mb:
			self._mb.RemoveHelp()

		if self._clearCurrentSelection:

			# Message to send when out of last selected item
			if self._selectedItem != -1:
				item = self._itemsArr[self._selectedItem]
				if item.IsSubMenu() and item.GetSubMenu().IsShown():
					return

				# Message to send when out of last selected item
				if self._selectedItem != -1:
					self.SendOverItem(self._selectedItem, False)

			oldSelection = self._selectedItem
			self._selectedItem = -1
			dc = wx.ClientDC(self)
			self.DrawSelection(dc, oldSelection)

		self._clearCurrentSelection = True

		if "__WXMSW__" in wx.Platform:
			self.SetCursor(self._oldCur)


	def OnMouseEnterWindow(self, event):
		""" Handles the wx.EVT_ENTER_WINDOW event for FlatMenu. """

		if "__WXMSW__" in wx.Platform:
			self._oldCur = self.GetCursor()
			self.SetCursor(wx.StockCursor(wx.CURSOR_ARROW))

		event.Skip()


	def OnKillFocus(self, event):
		""" Handles the wx.EVT_KILL_FOCUS event for FlatMenu. """

		self.Dismiss(True, True)


	def CloseSubMenu(self, itemIdx, alwaysClose=False):
		""" Close a child sub-menu. """

		item = None
		subMenu = None

		if itemIdx >= 0 and itemIdx < len(self._itemsArr):
			item = self._itemsArr[itemIdx]

		# Close sub-menu first
		if item:
			subMenu = item.GetSubMenu()

		if self._openedSubMenu:
			if self._openedSubMenu != subMenu or alwaysClose:
				# We have another sub-menu open, close it
				self._openedSubMenu.Dismiss(False, True)
				self._openedSubMenu = None


	def DoAction(self, itemIdx):
		""" Performs an action based on user selection. """

		if itemIdx < 0 or itemIdx >= len(self._itemsArr):
			raise Exception("Invalid menu item")
			return

		item = self._itemsArr[itemIdx]

		if not item.IsEnabled() or item.IsSeparator():
			return

		# Close sub-menu if needed
		self.CloseSubMenu(itemIdx)

		if item.IsSubMenu() and not item.GetSubMenu().IsShown():

			# Popup child menu
			self.TryOpenSubMenu(itemIdx)
			return

		if item.IsRadioItem():
			# if the radio item is already checked,
			# just send command event. Else, check it, uncheck the current
			# checked item in the radio item group, and send command event
			if not item.IsChecked():
				item._groupPtr.SetSelection(item)

		elif item.IsCheckable():

			item.Check(not item.IsChecked())
			dc = wx.ClientDC(self)
			self.DrawSelection(dc)

		if not item.IsSubMenu():

			self.Dismiss(True, False)

			# Send command event
			self.SendCmdEvent(itemIdx)


	def TryOpenSubMenu(self, itemIdx, selectFirst=False):
		""" If itemIdx is an item with submenu, open it. """

		if itemIdx < 0 or itemIdx >= len(self._itemsArr):
			return False

		item = self._itemsArr[itemIdx]
		if item.IsSubMenu() and not item.GetSubMenu().IsShown():

			pos = wx.Point()

			# Popup child menu
			pos.x = item.GetRect().GetWidth()+ item.GetRect().GetX()-5
			pos.y = item.GetRect().GetY()
			self._clearCurrentSelection = False
			self._openedSubMenu = item.GetSubMenu()
			item.GetSubMenu().Popup(pos, self._owner, self)

			# Select the first child
			if selectFirst:

				dc = wx.ClientDC(item.GetSubMenu())
				item.GetSubMenu()._selectedItem = 0
				item.GetSubMenu().DrawSelection(dc)

			return True

		return False


	def _RemoveById(self, id):
		""" Used internally. """

		# First we search for the menu item (recursivley)
		menuParent = None
		item = None
		idx = wx.NOT_FOUND
		idx, menuParent = self.FindMenuItemPos(id)

		if idx != wx.NOT_FOUND:

			# Remove the menu item
			item = menuParent._itemsArr[idx]
			menuParent._itemsArr.pop(idx)

			# update group
			if item._groupPtr and item.IsRadioItem():
				item._groupPtr.Remove(item)

			# Resize the menu
			menuParent.ResizeMenu()

		return item


	def Remove(self, item):
		""" Removes an item from the menu. """

		if type(item) != type(1):
			item = item.GetId()

		return self._RemoveById(item)


	def _DestroyById(self, id):
		""" Used internally. """

		item = None
		item = self.Remove(id)

		if item:
			del item


	def Destroy(self, item):
		""" Destroys an item from the menu. """

		if type(item) != type(1):
			item = item.GetId()

		self._DestroyById(item)


	def Insert(self, pos, id, item, helpString="", kind=wx.ITEM_NORMAL):
		""" Inserts an item into the menu. """

		newitem = FlatMenuItem(self, id, item, helpString, kind)
		return self.InsertItem(pos, newitem)


	def InsertItem(self, pos, item):
		""" Inserts an item into the menu. """

		if pos == len(self._itemsArr):
			# Append it
			return self.AppendItem(item)

		# Insert the menu item
		self._itemsArr.insert(pos, item)
		item._isAttachedToMenu = True

		# Recalculate the menu geometry
		self.ResizeMenu()

		# Update radio groups
		self.UpdateRadioGroup(item)

		return item


	def UpdateRadioGroup(self, item):
		""" Updates a group of radio items. """

		if item.IsRadioItem():

			# Udpate radio groups in case this item is a radio item
			sibling = self.GetSiblingGroupItem(item)
			if sibling:

				item._groupPtr = sibling._groupPtr
				item._groupPtr.Add(item)

				if item.IsChecked():

					item._groupPtr.SetSelection(item)

			else:

				# first item in group
				item._groupPtr = FlatMenuItemGroup()
				item._groupPtr.Add(item)
				item._groupPtr.SetSelection(item)


	def ResizeMenu(self):
		""" Resizes the menu to the correct size. """

		# can we do the resize?
		if not self._resizeMenu:
			return

		items = self._itemsArr
		self._itemsArr = []

		# Clear accelerator table
		self._accelArray = []

		# Reset parameters and menu size
		self._menuWidth =  2*self._marginWidth
		self._imgMarginX = 0
		self._markerMarginX = 0
		self._textX = self._marginWidth
		self._rightMarginPosX = -1
		self._itemHeight = self._marginHeight
		self.SetSize(wx.Size(self._menuWidth*self._numCols, self._itemHeight+4))

		# Now we simply add the items
		for item in items:
			self.AppendItem(item)


	def SetNumberColumns(self, numCols):
		""" Sets the number of columns for a menu window. """

		if self._numCols == numCols:
			return

		self._numCols = numCols
		self.ResizeMenu()
		self.Refresh()


	def GetNumberColumns(self):
		""" Returns the number of columns for a menu window. """

		return self._numCols


	def FindItem(self, itemId, menu=None):
		""" Finds an item inside the menu based on its id. """

		idx = wx.NOT_FOUND

		if menu:

			idx, menu = self.FindMenuItemPos(itemId, menu)
			if idx != wx.NOT_FOUND:
				return menu._itemsArr[idx]
			else:
				return None

		else:

			idx, parentMenu = self.FindMenuItemPos(itemId, None)
			if idx != wx.NOT_FOUND:
				return parentMenu._itemsArr[idx]
			else:
				return None


	def FindMenuItemPos(self, itemId, menu=None):
		""" Finds an item and its position inside the menu based on its id. """

		menu = None
		item = None

		idx = wx.NOT_FOUND

		for i in xrange(len(self._itemsArr)):

			item = self._itemsArr[i]

			if item.GetId() == itemId:

				menu = self
				idx = i
				break

			elif item.IsSubMenu():

				idx, menu = item.GetSubMenu().FindMenuItemPos(itemId, menu)
				if idx != wx.NOT_FOUND:
					break

			else:

				item = None

		return idx, menu


	def GetAccelTable(self):
		""" Returns the menu accelerator table. """

		n = len(self._accelArray)
		if n == 0:
			return wx.NullAcceleratorTable

		entries = [wx.AcceleratorEntry() for ii in xrange(n)]

		for counter in len(entries):
			entries[counter] = self._accelArray[counter]

		table = wx.AcceleratorTable(entries)
		del entries

		return table


	def GetAccelArray(self):

		return self._accelArray


	# events
	def SendCmdEvent(self, itemIdx):
		""" Actually sends menu command events. """

		if itemIdx < 0 or itemIdx >= len(self._itemsArr):
			raise Exception("Invalid menu item")
			return

		item = self._itemsArr[itemIdx]
		
		# Create the event
		event = wx.CommandEvent(wxEVT_FLAT_MENU_SELECTED, item.GetId())

		# For checkable item, set the IsChecked() value
		if item.IsCheckable():
			event.SetInt((item.IsChecked() and [1] or [0])[0])

		event.SetEventObject(self)

		if self._owner:
			self._owner.GetEventHandler().ProcessEvent(event)
		else:
			self.GetEventHandler().ProcessEvent(event)


	def SendOverItem(self, itemIdx, over):
		""" Sends a mouse over/mouse out event on menu items. """

		item = self._itemsArr[itemIdx]

		# Create the event
		event = FlatMenuEvent((over and [wxEVT_FLAT_MENU_ITEM_MOUSE_OVER] or [wxEVT_FLAT_MENU_ITEM_MOUSE_OUT])[0], item.GetId())

		# For checkable item, set the IsChecked() value
		if item.IsCheckable():
			event.SetInt((item.IsChecked() and [1] or [0])[0])

		event.SetEventObject(self)

		if self._owner:
			self._owner.GetEventHandler().ProcessEvent(event)
		else:
			self.GetEventHandler().ProcessEvent(event)


	def SendUIEvent(self, itemIdx):
		""" Actually sends menu UI events. """

		if itemIdx < 0 or itemIdx >= len(self._itemsArr):
			raise Exception("Invalid menu item")
			return

		item = self._itemsArr[itemIdx]
		event = wx.UpdateUIEvent(item.GetId())

		event.Check(item.IsChecked())
		event.Enable(item.IsEnabled())
		event.SetText(item.GetText())
		event.SetEventObject(self)

		if self._owner:
			self._owner.GetEventHandler().ProcessEvent(event)
		else:
			self.GetEventHandler().ProcessEvent(event)

		item.Check(event.GetChecked())
		item.SetLabel(event.GetText())
		item.Enable(event.GetEnabled())


	def Clear(self):
		""" Clears the menu items. """

		# since Destroy() call ResizeMenu(), we turn this flag on
		# to avoid resizing the menu for every item removed
		self._resizeMenu = False

		lenItems = len(self._itemsArr)
		for ii in xrange(lenItems):
			self.Destroy(self._itemsArr[0].GetId())

		# Now we can resize the menu
		self._resizeMenu = True
		self.ResizeMenu()


	def FindMenuItemPosSimple(self, item):
		""" Finds an item and its position inside the menu based on its id. """

		if item == None or len(self._itemsArr) == 0:
			return wx.NOT_FOUND

		for i in xrange(len(self._itemsArr)):
			if self._itemsArr[i] == item:
				return i

		return wx.NOT_FOUND


	def GetAllItems(self, menu=None, items=[]):
		""" Internal function to help recurse thru all over the menu items. """

		# first copy the current menu items
		newitems = [item for item in items]

		if not menu:
			return newitems

		# if any item in this menu has sub-menu, copy them as well
		for i in xrange(len(menu._itemsArr)):
			if menu._itemsArr[i].IsSubMenu():
				newitems = self.GetAllItems(menu._itemsArr[i].GetSubMenu(), newitems)

		return newitems


	def GetSiblingGroupItem(self, item):
		""" Used internally. """

		pos = self.FindMenuItemPosSimple(item)
		if pos in [wx.NOT_FOUND, 0]:
			return None

		if self._itemsArr[pos-1].IsRadioItem():
			return self._itemsArr[pos-1]

		return None


	def ScrollDown(self):
		""" Scrolls the menu down (for very tall menus). """

		# increase the self._from index
		if not self._itemsArr[-1].IsShown():
			self._first += 1
			self.Refresh()

			return True

		else:
			if self._downButton:
				self._downButton.GetTimer().Stop()

			return False


	def ScrollUp(self):
		""" Scrolls the menu up (for very tall menus). """

		if self._first == 0:
			if self._upButton:
				self._upButton.GetTimer().Stop()

			return False

		else:

			self._first -= 1
			self.Refresh()
			return True


	# Not used anymore
	def TryScrollButtons(self, event):
		""" Used internally. """

		return False


	def OnTimer(self, event):
		""" Handles the wx.EVT_TIMER event for FlatMenu. """

		if self._upButton and self._upButton.GetTimerId() == event.GetId():

			self.ScrollUp()

		elif self._downButton and self._downButton.GetTimerId() == event.GetId():

			self.ScrollDown()

		else:

			event.Skip()


#--------------------------------------------------------
# Class MenuKbdRedirector
#--------------------------------------------------------

class MenuKbdRedirector(wx.EvtHandler):
	""" A keyboard event handler. """

	def __init__(self, menu, oldHandler):
		""" Default class constructor. """

		self._oldHandler = oldHandler
		self.SetMenu(menu)
		wx.EvtHandler.__init__(self)


	def SetMenu(self, menu):
		""" Sets the listened menu. """

		self._menu = menu


	def ProcessEvent(self, event):
		""" Processes the inout event. """

		if event.GetEventType() in [wx.EVT_KEY_DOWN, wx.EVT_CHAR, wx.EVT_CHAR_HOOK]:
			return self._menu.OnChar(event.GetKeyCode())
		else:
			return self._oldHandler.ProcessEvent(event)


#--------------------------------------------------------
# Class FocusHandler
#--------------------------------------------------------

class FocusHandler(wx.EvtHandler):
	""" A focus event handler. """

	def __init__(self, menu):
		""" Default class constructor. """

		wx.EvtHandler.__init__(self)
		self.SetMenu(menu)

		self.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
		self.Bind(wx.EVT_KILL_FOCUS, self.OnKillFocus)


	def SetMenu(self, menu):
		""" Sets the listened menu. """

		self._menu = menu


	def OnKeyDown(self, event):
		""" Handles the wx.EVT_KEY_DOWN event for FocusHandler. """

		# Let parent process it
		self._menu.OnKeyDown(event)


	def OnKillFocus(self, event):
		""" Handles the wx.EVT_KILL_FOCUS event for FocusHandler. """

		wx.PostEvent(self._menu, event)


